@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/bkserverd.c,v 1.136 2012/02/29 10:07:26 valtri Exp $"
+#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/bkserverd.c,v 1.144 2012/08/14 14:29:58 zsustr Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -185,6 +185,8 @@ struct _edg_wll_authz_policy	authz_policy = { NULL, 0};
 static int 		exclusive_zombies = 1;
 static char		**msg_brokers = NULL;
 static char		**msg_prefixes = NULL;
+char *		html_header = NULL;
+static int	html_header_forced = 0;
 
 
 static struct option opts[] = {
@@ -211,6 +213,7 @@ static struct option opts[] = {
 	{"super-user",	1, NULL,	'R'},
 //	{"super-users-file",	1, NULL,'F'},
 	{"msg-conf",	1, NULL,'F'},
+	{"html-header",	1, NULL,'H'},
 	{"no-index",	1, NULL,	'x'},
 	{"strict-locking",0, NULL,	'O'},
 	{"limits",	1, NULL,	'L'},
@@ -239,7 +242,7 @@ static struct option opts[] = {
 	{NULL,0,NULL,0}
 };
 
-static const char *get_opt_string = "Ac:k:C:V:p:a:drm:ns:i:S:D:J:jR:F:xOL:N:X:Y:T:t:e:f:zb:gPBo:q:W:Z:GI:l:E"
+static const char *get_opt_string = "Ac:k:C:V:p:a:drm:ns:i:S:D:J:jR:F:xOL:N:X:Y:T:t:e:f:zb:gPBo:q:W:Z:GI:l:EH:"
 #ifdef GLITE_LB_SERVER_WITH_WS
 	"w:"
 #endif
@@ -302,6 +305,7 @@ static void usage(char *me)
 		"\t-l,--policy\tauthorization policy file\n"
 		"\t-E,--exclusive-zombies-off\twith 'exclusive' flag, allow reusing IDs of purged jobs\n"
 		"\t-F,--msg-conf\t path to configuration file with messaging settings\n"
+		"\t-H,--html-header\t path to HTML header file for customized/branded HTML output\n"
 
 	,me);
 }
@@ -425,6 +429,8 @@ int main(int argc, char *argv[])
 	purge_timeout[EDG_WLL_JOB_CLEARED] = 60*60*24*3;
 	purge_timeout[EDG_WLL_JOB_ABORTED] = 60*60*24*7;
 	purge_timeout[EDG_WLL_JOB_CANCELLED] = 60*60*24*7;
+
+
 	if (edg_wll_InitContext(&ctx) != 0) {
 		fprintf(stderr, "Couldn't create L&B context, exiting.\n");
 		exit(1);
@@ -479,6 +485,7 @@ int main(int argc, char *argv[])
 		case 'i': strcpy(pidfile,optarg); pidfile_forced = 1; break;
 		case 'R': add_root(ctx, optarg, ADMIN_ACCESS); break;
 		case 'F': msg_conf = strdup(optarg); break;
+		case 'H': html_header = strdup(optarg); html_header_forced = 1; break;
 		case 'x': noIndex = atoi(optarg);
 			  if (noIndex < 0 || noIndex > 2) { usage(name); return 1; }
 			  break;
@@ -578,6 +585,21 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (!html_header) {
+		char *html_header_prefix = getenv("GLITE_LB_LOCATION_ETC");
+		if (!html_header_prefix) html_header_prefix="/etc/glite-lb";
+		asprintf(&html_header, "%s/html-header.html", html_header_prefix);
+	}
+	if (html_header) {
+		FILE *fp = fopen(html_header, "r");
+		if( fp ) {
+			fclose(fp);
+			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_DEBUG, "Using HTML header file %s", html_header);
+		} else {
+			glite_common_log(LOG_CATEGORY_CONTROL, html_header_forced ? LOG_PRIORITY_ERROR : LOG_PRIORITY_INFO, "Cannot open HTML header file %s", html_header);
+		}
+	}
+
 	if (msg_conf) {
 		int retv_msg;
 		glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_DEBUG, "Parsing MSG conf file: %s", msg_conf);
@@ -678,7 +700,7 @@ int main(int argc, char *argv[])
 		ret = edg_wll_gss_watch_creds(server_cert, &cert_mtime);
 		if (ret < 0)
         		glite_common_log(LOG_CATEGORY_SECURITY,LOG_PRIORITY_WARN,"edg_wll_gss_watch_creds failed, unable to access credentials\n");
-		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &mycred, &gss_code) )
+		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &mycred, &gss_code) && mycred->name != NULL)
 		{
 			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_INFO, "Server identity: %s", mycred->name);
 			server_subject = strdup(mycred->name);
@@ -689,7 +711,6 @@ int main(int argc, char *argv[])
 			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_ERROR, "Server running unauthenticated");
 			server_subject = strdup("anonymous LB");
 		}
-
 		if ( noAuth ) 
 			glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_INFO, "Server in promiscuous mode");
 		glite_common_log(LOG_CATEGORY_CONTROL, LOG_PRIORITY_INFO, 
@@ -772,7 +793,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (!dbstring) dbstring = getenv("LBDB");
-	if (!dbstring) dbstring = strdup(DEFAULTCS);
+	if (!dbstring) dbstring = DEFAULTCS;
 		
 	/* Just check the database and let it be. The slaves do the job. */
 	if (wait_for_open(ctx, dbstring)) {
@@ -878,7 +899,6 @@ int main(int argc, char *argv[])
         if (port) free(port);
 	edg_wll_gss_release_cred(&mycred, NULL);
 
-
 	return 0;
 }
 
@@ -1003,7 +1023,7 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 	int			npref, totpref;
 
 
-
+#if 0
 	switch ( edg_wll_gss_watch_creds(server_cert, &cert_mtime) ) {
 	case 0: break;
 	case 1:
@@ -1019,6 +1039,15 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 		glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_ERROR, "[%d] edg_wll_gss_watch_creds failed, unable to access credentials", getpid());
 		break;
 	}
+#else
+		if ( !edg_wll_gss_acquire_cred_gsi(server_cert, server_key, &newcred, &gss_code) ) {
+			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_INFO, "[%d] reloading credentials successful", getpid());
+			edg_wll_gss_release_cred(&mycred, NULL);
+			mycred = newcred;
+		} else { 
+			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_WARN, "[%d] reloading credentials failed, using old ones", getpid());
+		}
+#endif
 
 	if ( edg_wll_InitContext(&ctx) )
 	{
@@ -1075,6 +1104,7 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 	ctx->rssTime = rss_time;
 
 	if (policy_file) ctx->authz_policy_file = strdup(policy_file);
+	if (html_header) ctx->html_header_file = strdup(html_header);
 
 	gettimeofday(&conn_start, 0);
 
@@ -1199,6 +1229,7 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 		glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_INFO, "[%d] client DN: %s",getpid(),ctx->peerName);
 	}
 
+#if 0
 	if ( edg_wll_SetVomsGroups(ctx, &ctx->connections->serverConnection->gss, server_cert, server_key, vomsdir, cadir) )
 	{
 		char *errt, *errd;
@@ -1208,13 +1239,15 @@ int bk_handle_connection(int conn, struct timeval *timeout, void *data)
 		free(errt); free(errd);
 		edg_wll_ResetError(ctx); 
 	}
+#endif
+
 	if (ctx->vomsGroups.len > 0)
 	{
 		int i;
   
 		 glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_DEBUG, "[%d] client's VOMS groups:",getpid());
 		for ( i = 0; i < ctx->vomsGroups.len; i++ )
-			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_DEBUG, "\t%s:%s", ctx->vomsGroups.val[i].vo, ctx->vomsGroups.val[i].name);
+			glite_common_log(LOG_CATEGORY_SECURITY, LOG_PRIORITY_DEBUG, "\t %s:%s", ctx->vomsGroups.val[i].vo, ctx->vomsGroups.val[i].name);
 	}
 	if (ctx->fqans && *(ctx->fqans))
 	{
