@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_authz.c,v 1.49.2.4 2012/05/16 13:12:26 zsustr Exp $"
+#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_authz.c,v 1.52 2012/06/22 12:57:28 valtri Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -23,8 +23,12 @@ limitations under the License.
 
 #include <openssl/x509.h>
 #include <gssapi.h>
+#ifndef NO_GLOBUS_GSSAPI
+#ifdef HAVE_LCAS
 #include "lcas/lcas_pem.h"
+#endif
 #include "voms/voms_apic.h"
+#endif
 #include "glite/lb/context-int.h"
 #include "lb_authz.h"
 #include "glite/lbu/log.h"
@@ -61,6 +65,7 @@ edg_wll_get_fqans(edg_wll_Context ctx, struct vomsdata *voms_info,
    attrs = NULL;
    num = 0;
 
+#ifndef NO_GLOBUS_GSSAPI
    for (voms_cert = voms_info->data; voms_cert && *voms_cert; voms_cert++) {
       for (f = (*voms_cert)->fqan; f && *f; f++) {
          tmp = realloc(attrs, (num + 1) * sizeof(*attrs));
@@ -81,11 +86,13 @@ edg_wll_get_fqans(edg_wll_Context ctx, struct vomsdata *voms_info,
       attrs = tmp;
       attrs[num++] = NULL;
    }
+#endif
    
    *fqans = attrs;
    return 0;
 }
 
+#ifndef NO_GLOBUS_GSSAPI
 static int
 add_groups(edg_wll_Context ctx, struct voms *voms_cert, char *vo_name,
       	   edg_wll_VomsGroups *groups)
@@ -136,6 +143,7 @@ get_groups(edg_wll_Context ctx, struct vomsdata *voms_info,
    res_groups->val = groups.val;
    return 0;
 }
+#endif
 
 int
 edg_wll_SetVomsGroups(edg_wll_Context ctx, edg_wll_GssConnection *gss, char *server_cert, char *server_key, char *voms_dir, char *ca_dir)
@@ -151,6 +159,10 @@ edg_wll_SetVomsGroups(edg_wll_Context ctx, edg_wll_GssConnection *gss, char *ser
    memset (&ctx->vomsGroups, 0, sizeof(ctx->vomsGroups));
    edg_wll_ResetError(ctx);
 
+#ifdef NO_GLOBUS_GSSAPI
+   return 0;
+#else
+
    if (ctx->fqans) {
       char **f;
       for (f = ctx->fqans; f && *f; f++)
@@ -175,7 +187,6 @@ edg_wll_SetVomsGroups(edg_wll_Context ctx, edg_wll_GssConnection *gss, char *ser
       ret = -1; /* XXX VOMS Error */
       goto end;
    }
-
    ret = VOMS_RetrieveFromCtx(gss->context, RECURSE_CHAIN, voms_info, &err);
    if (ret == 0) {
       if (err == VERR_NOEXT)
@@ -203,8 +214,11 @@ end:
       VOMS_Destroy(voms_info);
 
    return ret;
+
+#endif /* NO_GLOBUS_GSSAPI */
 }
 
+#ifndef NO_GLOBUS_GSSAPI
 void
 edg_wll_FreeVomsGroups(edg_wll_VomsGroups *groups)
 {
@@ -220,6 +234,7 @@ edg_wll_FreeVomsGroups(edg_wll_VomsGroups *groups)
 	 free(groups->val[len].name);
    }
 }
+#endif
 
 static int
 parse_creds(edg_wll_Context ctx, edg_wll_VomsGroups *groups, char **fqans,
@@ -950,6 +965,8 @@ check_store_authz(edg_wll_Context ctx, edg_wll_Event *ev)
 	     break;
    }
 
+#ifndef NO_GLOBUS_GSSAPI
+#ifdef HAVE_LCAS
    if (enable_lcas) {
       /* XXX make a real RSL ? */
       request = (char *) action2name(action);
@@ -965,6 +982,8 @@ check_store_authz(edg_wll_Context ctx, edg_wll_Event *ev)
       if (ret)
 	 goto end;
    }
+#endif
+#endif
 
    /* by default the server is open to any authenticated client */
    if (policy_file == NULL)
