@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_html.c,v 1.38.2.4 2012/06/15 15:30:05 valtri Exp $"
+#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_html.c,v 1.47 2012/08/10 15:12:26 zsustr Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -42,6 +42,23 @@ limitations under the License.
 #define UNUSED_VAR
 #endif
 
+char *get_html_header(edg_wll_Context ctx, int text) {
+	char *header = NULL;
+	size_t header_len = 0, rlen = 0;
+	FILE *header_file;
+
+	if (text) return NULL;
+
+	if ((header_file = fopen(ctx->html_header_file, "r"))) { 
+		rlen = getdelim( &header, &header_len, '\0', header_file);
+		fclose (header_file);
+	}
+	
+	if (rlen == -1 ) header=strdup("<style type=\"text/css\">tr.notused {color: gray; text-align: left;}</style>");
+
+	return header;
+}
+
 int edg_wll_QueryToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_Event *eventsOut UNUSED_VAR, char **message UNUSED_VAR)
 {
 /* not implemented yet */
@@ -61,10 +78,9 @@ int jobstat_cmp (const void *a, const void *b) {
 /* construct Message-Body of Response-Line for edg_wll_UserJobs */
 int edg_wll_UserInfoToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wlc_JobId *jobsOut, edg_wll_JobStat *statsOut, char **message, int text)
 {
-        char *pomA = NULL, *pomB, *pomC;
-	int i, total = 0, bufsize, written = 0, linlen, wassub = 0, lineoverhead;
+        char *pomA = NULL, *pomB, *pomC, *header = NULL, *recent_parent = NULL;
+	int i, total = 0, bufsize, written = 0, linlen, wassub = 0, issub = 0, lineoverhead;
 	JobIdSorter *order;
-	
 
         while (jobsOut && jobsOut[total]) total++;
 
@@ -84,9 +100,13 @@ int edg_wll_UserInfoToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wlc_JobId *jobsOu
 	}
 	else {		
 		qsort(order, total, sizeof(JobIdSorter), jobstat_cmp); 
-	
-		linlen = asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>User Jobs</TITLE>\n<BODY>\n"
-			"<h2><B>User Jobs</B></h2>\nTotal of %d<P><UL>\n",total);
+
+		header = get_html_header(ctx, text);
+
+		linlen = asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>User Jobs</TITLE>\n<HEAD>\n%s\n</HEAD>\n<BODY>\n"
+			"<h2><B>User Jobs</B></h2>\nTotal of %d<P><UL>\n", header ? header : "", total);
+
+		free(header);
 
 		asprintf(&pomC, "</UL>User subject: %s<p>\n"
 			"</BODY>\n</HTML>",
@@ -106,11 +126,12 @@ int edg_wll_UserInfoToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wlc_JobId *jobsOu
 	for (i = 0; i < total; i++) {
 		if (text) linlen = asprintf(&pomA,"%s%s", order[i].id_unparsed, i + 1 == total ? "\n" : ",");
 		else {
+			issub = order[i].parent_unparsed && recent_parent && !strcmp(recent_parent, order[i].parent_unparsed) ? 1 : 0;
 			linlen = asprintf(&pomA, "%s<li><a href=\"%s\">%s</a></li>\n",
-				order[i].parent_unparsed ? (wassub++ ? "" : "<ul>") : (wassub ? "</UL>" : ""),
+				issub ? (wassub++ ? "" : "<ul>") : (wassub ? "</UL>" : ""),
 				order[i].id_unparsed,
 				order[i].id_unparsed );
-			if (!order[i].parent_unparsed) wassub = 0;
+			if (!issub || !order[i].parent_unparsed) { wassub = 0; recent_parent = order[i].id_unparsed; }
 		}
 
 		while (written+linlen >= bufsize) {
@@ -144,7 +165,7 @@ int edg_wll_UserInfoToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wlc_JobId *jobsOu
 
 int edg_wll_UserNotifsToHTML(edg_wll_Context ctx UNUSED_VAR, char **notifids, char **message, http_admin_option option, int adm) {
 	char *pomA = NULL, *pomB = NULL;
-	char *mylink = NULL, *alllink = NULL, *foreignlink = NULL, *heading = NULL;
+	char *mylink = NULL, *alllink = NULL, *foreignlink = NULL, *heading = NULL, *header = NULL;
         pomB = strdup("");
 
 	int i = 0;
@@ -174,24 +195,27 @@ int edg_wll_UserNotifsToHTML(edg_wll_Context ctx UNUSED_VAR, char **notifids, ch
 	}
 	if (!heading) asprintf(&heading,"Your notifications");
 
+	header = get_html_header(ctx, 0);
 	char *ret;
-        asprintf(&ret, "<html>\r\n\t<body>\r\n");
-	asprintf(&ret, "<html>\r\n\t<body>\r\n"
+	asprintf(&ret, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Notifications</TITLE>\n<HEAD>\n<HEAD>\n%s\n</HEAD>\n<body>\r\n"
 			"<h2><B>%s</B></h2>\r\n"
 			"<P>%s%s%s"
+			"<P>Total of %d"
                         "<ul>%s</ul>"
-			"\t</body>\r\n</html>",
+			"\t</body>\r\n</HTML>",
+			header,
 			heading,
 			mylink ? mylink : "",
 			alllink ? alllink : "",
 			foreignlink ? foreignlink : "",
-                        pomA ? pomA : "No registrations found"
+                        i, pomA ? pomA : "No registrations found"
         );
         free(pomA);
 	free(mylink);
 	free(alllink);
 	free(foreignlink);
 	free(heading);
+	free(header);
 
 	*message = ret;
 
@@ -203,11 +227,11 @@ int edg_wll_UserNotifsToHTML(edg_wll_Context ctx UNUSED_VAR, char **notifids, ch
 	int l; \
 	if ((field) != (null)){ \
 		l = asprintf(&pomA,"<tr><th align=\"left\">" name ":</th>" \
-			"<td>" type "</td></tr>", (field)); \
+			"<td>" type "</td></tr>\n", (field)); \
 	} \
 	else{ \
-		l = asprintf(&pomA,"<tr><th align=\"left\"><span style=\"color:grey\">" name \
-			"</span></th></tr>"); \
+                l = asprintf(&pomA,"<tr class=\"notused\"><th>" name \
+                        "</th></tr>\n"); \
 	} \
 	pomB = realloc(pomB, sizeof(*pomB)*(pomL+l+1)); \
 	strcpy(pomB+pomL, pomA); \
@@ -221,11 +245,11 @@ int edg_wll_UserNotifsToHTML(edg_wll_Context ctx UNUSED_VAR, char **notifids, ch
         int l; \
         if ((field) != (null)){ \
                 l = asprintf(&pomA,"<tr><th align=\"left\">" name ":</th>" \
-                        "<td><a href=\""type"\">" type "</a></td></tr>", (field), (field)); \
+                        "<td><a href=\""type"\">" type "</a></td></tr>\n", (field), (field)); \
         } \
         else{ \
-                l = asprintf(&pomA,"<tr><th align=\"left\"><span style=\"color:grey\">" name \
-                        "</span></th></tr>"); \
+                l = asprintf(&pomA,"<tr class=\"notused\"><th>" name \
+                        "</th></tr>\n"); \
         } \
         pomB = realloc(pomB, sizeof(*pomB)*(pomL+l+1)); \
         strcpy(pomB+pomL, pomA); \
@@ -235,7 +259,7 @@ int edg_wll_UserNotifsToHTML(edg_wll_Context ctx UNUSED_VAR, char **notifids, ch
 }
 
 int edg_wll_NotificationToHTML(edg_wll_Context ctx UNUSED_VAR, notifInfo *ni, char **message){
-	char *pomA = NULL, *pomB = NULL, *flags = NULL, *cond;
+	char *pomA = NULL, *pomB = NULL, *flags = NULL, *cond, *header = NULL;
 	int pomL = 0;
 
 	flags = edg_wll_stat_flags_to_string(ni->flags);
@@ -245,6 +269,11 @@ int edg_wll_NotificationToHTML(edg_wll_Context ctx UNUSED_VAR, notifInfo *ni, ch
 	TR("Destination", "%s", ni->destination, NULL);
 	TR("Valid until", "%s", ni->valid, NULL);
 	TR("Flags", "%s", flags, NULL);
+	if (strcmp(ni->jobid,"all_jobs")) {
+		TRL("Job ID", "%s", ni->jobid, NULL); }
+	else {
+		TR("Job ID", "%s", "&mdash;", NULL);
+	}
         free(flags);
 
 	if (! edg_wll_Condition_Dump(ni, &cond, 0)){
@@ -257,11 +286,14 @@ int edg_wll_NotificationToHTML(edg_wll_Context ctx UNUSED_VAR, notifInfo *ni, ch
 	}
 	free(cond);
 
-	asprintf(&pomA, "<html>\r\n\t<body>\r\n"
+	header = get_html_header(ctx, 0);
+	asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Notification Detail</TITLE>\n<HEAD>\n<HEAD>\n%s\n</HEAD>\n<body>\r\n"
 		"<h2>Notification %s</h2>\r\n"
 		"<table halign=\"left\">%s</table>"
 		"\t</body>\r\n</html>",
+		header,
 		ni->notifid, pomB);
+	free(header);
 	
 	*message = pomA;
 
@@ -273,7 +305,7 @@ int edg_wll_GeneralJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobSt
 {
         char *pomA = NULL, *pomB = NULL;
 	int pomL = 0;
-	char	*chid,*chstat,*chis = NULL, *chos = NULL, *chpa = NULL;
+	char	*chid,*chstat,*chis = NULL, *chos = NULL, *chpa = NULL, *header = NULL;
 	char	*jdl,*rsl,*children;
 	int	i;
 
@@ -344,13 +376,16 @@ int edg_wll_GeneralJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobSt
 		}
 	}
 
-        asprintf(&pomA, "<html>\r\n\t<body>\r\n"
+	header = get_html_header(ctx, 0);
+	asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Job Detail</TITLE>\n<HEAD>\n<HEAD>\n%s\n</HEAD>\n<body>\r\n"
 			"<h2>%s</h2>\r\n"
 			"<table halign=\"left\">%s</table>"
 			"%s%s%s"
 			"\t</body>\r\n</html>",
+			header,
                 	chid,pomB,jdl,rsl, children);
         free(pomB);
+	free(header);
 
         *message = pomA;
 
@@ -366,7 +401,7 @@ int edg_wll_GeneralJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobSt
 
 int edg_wll_CreamJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobStat stat, char **message)
 {
-	char *chid, *pomA = NULL, *pomB = NULL, *jdl;
+	char *chid, *pomA = NULL, *pomB = NULL, *jdl, *header = NULL;
 	char *lbstat, *creamstat;
 	int pomL = 0;
 
@@ -419,11 +454,13 @@ int edg_wll_CreamJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobStat
                         asprintf(&jdl,"<h3>Job description (not a ClassAd)"
                                 "</h3>\r\n<pre>%s</pre>\r\n",stat.cream_jdl);
         }
-	 asprintf(&pomA, "<html>\r\n\t<body>\r\n"
+	header = get_html_header(ctx, 0);
+	asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Job Detail</TITLE>\n<HEAD>\n<HEAD>\n%s\n</HEAD>\n<body>\r\n"
                         "<h2>%s</h2>\r\n"
                         "<table halign=\"left\">%s</table>"
                         "%s"
                         "\t</body>\r\n</html>",
+			header,
                         chid,pomB,jdl);
         free(pomB); free(jdl);
 
@@ -434,7 +471,7 @@ int edg_wll_CreamJobStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobStat
 
 int edg_wll_FileTransferStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_JobStat stat, char **message)
 {
-        char *pomA = NULL, *pomB = NULL, *lbstat, *children;
+        char *pomA = NULL, *pomB = NULL, *lbstat, *children, *header = NULL;
         int pomL = 0, i;
         char    *chid,*chcj,*chpar,*chsbt=NULL;
 
@@ -473,14 +510,16 @@ int edg_wll_FileTransferStatusToHTML(edg_wll_Context ctx UNUSED_VAR, edg_wll_Job
                         children = pomA;
                 }
 	}
-
-        asprintf(&pomA, "<html>\r\n\t<body>\r\n"
+	header = get_html_header(ctx, 0);
+	asprintf(&pomA, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Job Detail</TITLE>\n<HEAD>\n<HEAD>\n%s\n</HEAD>\n<body>\r\n"
                         "<h2>%s</h2>\r\n"
                         "<table halign=\"left\">%s</table>\r\n"
 			"%s\n"
                         "\t</body>\r\n</html>",
+			header,
                         chid, pomB, children);
         free(pomB);
+	free(header);
 
         *message = pomA;
 
@@ -514,7 +553,7 @@ static int replace_substr(char **text, char *s1, char *s2){
 
 int edg_wll_WSDLOutput(edg_wll_Context ctx UNUSED_VAR, char **message, char *filename){
 	FILE *f;
-	char *wsdl, *p, *start;
+	char *wsdl/*, *p, *start*/; // unused vars commented out
 	int i, b;
 
 	if ((f = fopen(filename, "r")) == NULL){
@@ -547,28 +586,32 @@ int edg_wll_WSDLOutput(edg_wll_Context ctx UNUSED_VAR, char **message, char *fil
 }
 
 int edg_wll_StatisticsToHTML(edg_wll_Context ctx, char **message, int text) {
-        char *out;
+        char *out, *header = NULL;
 	char* times[SERVER_STATISTICS_COUNT];
 	int i;
 	char *out_tmp;
 	time_t *starttime;
 
 	for (i = 0; i < SERVER_STATISTICS_COUNT; i++)
-		if (starttime = edg_wll_ServerStatisticsGetStart(ctx, i)) 
+		if ((starttime = edg_wll_ServerStatisticsGetStart(ctx, i))) 
 			if (text) asprintf(&(times[i]), "%ld", *starttime);
 			else times[i] = strdup((const char*)ctime(starttime));
 		else
 			times[i] = NULL;
 
-	if (!text) asprintf(&out, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Server Usage Statistics</TITLE>\n<BODY>\n"
+	header = get_html_header(ctx, text);
+	if (!text) asprintf(&out, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<TITLE>Server Usage Statistics</TITLE>\n<HEAD>%s\n</HEAD>\n<BODY>\n"
 		"<h2>LB Server Usage Statistics</h2>\n"
 		"%s"
 		"<table halign=\"left\">\n"
 		"<tr><td>Variable</td><td>Value</td><td>Measured from</td></tr>\n",
+		header ? header : "",
 		edg_wll_ServerStatisticsInTmp() ? "<b>WARNING: L&B statistics are stored in /tmp, please, configure L&B server to make them really persistent!</b><br/><br/>\n" : "");
 	else
 		asprintf(&out, "");
 
+	free(header);
+
 	for (i = 0; i < SERVER_STATISTICS_COUNT; i++) {
 		asprintf(&out_tmp, text ? "%s%s=%i,%s\n" : "%s<tr><td>%s</td><td>%i</td><td>%s</td></tr>\n", out,
 			text ? edg_wll_server_statistics_type_key[i] : edg_wll_server_statistics_type_title[i],
@@ -591,16 +634,48 @@ int edg_wll_StatisticsToHTML(edg_wll_Context ctx, char **message, int text) {
 	return 0;
 }
 
+int edg_wll_VMHostToHTML(edg_wll_Context ctx UNUSED_VAR, char *hostname, edg_wll_JobStat *states, char **message){
+        char *pomA = NULL, *pomB = NULL;
+	int i;
+
+        if (states) for (i = 0; states[i].state != EDG_WLL_JOB_UNDEF; i++){
+		char *status = edg_wll_VMStatToString(states[i].vm_state);
+		char *chid = edg_wlc_JobIdUnparse(states[i].jobId);
+		if (pomB)
+			asprintf(&pomA, "%s\t\t <li> <a href=\"%s\">%s</a> (%s)\r\n",
+				pomB, chid, chid, status);
+		else
+			asprintf(&pomA, "<li> <a href=\"%s\">%s</a> (%s)\r\n",
+                                chid, chid, status);
+		free(pomB);
+		pomB = pomA;
+                free(chid);
+		free(status);
+        }
+
+	asprintf(&pomA, "<html>\r\n\t<body>\r\n"
+                        "<h2>%s</h2>\r\n"
+                        "<table halign=\"left\">%s</table>"
+                        "\t</body>\r\n</html>",
+                        hostname,pomB);
+        free(pomB);
+
+        *message = pomA;
+
+        return 0;
+}
+
 char *edg_wll_ErrorToHTML(edg_wll_Context ctx,int code)
 {
-	char	*out,*et,*ed;
+	char	*out,*et,*ed,*header = NULL;
 	char	*msg = edg_wll_HTTPErrorMessage(code);
 	edg_wll_ErrorCode	e;
 
 	e = edg_wll_Error(ctx,&et,&ed);
-	asprintf(&out,"<html><head><title>Error</title></head>\n"
+	header = get_html_header(ctx, 0);
+	asprintf(&out, "<HTML>\n<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n<HEAD>\n<TITLE>Error</TITLE>\n%s\n</HEAD>\n"
 		"<body><h1>%s</h1>\n"
-		"%d: %s (%s)</body></html>",msg,e,et,ed);
+		"%d: %s (%s)</body></html>",header,msg,e,et,ed);
 
 	free(et); free(ed);
 	return out;
