@@ -1,4 +1,4 @@
-#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_proto.c,v 1.60.2.4 2012/05/31 15:42:00 zsustr Exp $"
+#ident "$Header: /cvs/jra1mw/org.glite.lb.server/src/lb_proto.c,v 1.71 2012/08/09 14:46:18 zsustr Exp $"
 /*
 Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 See http://www.eu-egee.org/partners for details on the copyright holders.
@@ -34,6 +34,7 @@ limitations under the License.
 #include "glite/lb/xml_conversions.h"
 #include "glite/jobid/strmd5.h"
 #include "glite/lbu/trio.h"
+#include "glite/lbu/escape.h"
 #include "glite/lbu/log.h"
 
 #include "lb_proto.h"
@@ -88,6 +89,13 @@ static const char* const response_headers_html[] = {
         NULL
 };
 
+static const char* const response_headers_plain[] = {
+        "Cache-Control: no-cache",
+        "Server: edg_wll_Server/" PROTO_VERSION "/" COMP_PROTO,
+        "Content-Type: text/plain",
+        NULL
+};
+
 volatile sig_atomic_t purge_quit = 0;
 
 
@@ -277,14 +285,14 @@ err:
 static int getNotifInfo(edg_wll_Context ctx, char *notifId, notifInfo *ni){
 	char *q = NULL;
         glite_lbu_Statement notif = NULL;
-	char *notifc[5] = {NULL, NULL, NULL, NULL, NULL};
+	char *notifc[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
 
 	char *can_peername = NULL;
 	int retval = HTTP_OK;
 
-	trio_asprintf(&q, "select n.destination, n.valid, n.conditions, n.flags, u.cert_subj "
-                "from notif_registrations as n, users as u "
-                "where (n.notifid='%s') AND (n.userid=u.userid)", notifId);
+	trio_asprintf(&q, "select n.destination, n.valid, n.conditions, n.flags, u.cert_subj, j.jobid "
+                "from notif_registrations as n, users as u, notif_jobs as j "
+                "where (n.notifid='%s') AND (n.userid=u.userid) AND (n.notifid=j.notifid)", notifId);
 	glite_common_log_msg(LOG_CATEGORY_LB_SERVER_DB, LOG_PRIORITY_DEBUG, q);
 	if (edg_wll_ExecSQL(ctx, q, &notif) < 0) return HTTP_INTERNAL;
         free(q); q = NULL;
@@ -301,6 +309,7 @@ static int getNotifInfo(edg_wll_Context ctx, char *notifId, notifInfo *ni){
 			parseJobQueryRec(ctx, notifc[2], strlen(notifc[2]), &(ni->conditions));
 			ni->flags = atoi(notifc[3]);
 			ni->owner = notifc[4];
+			ni->jobid = notifc[5];
 		}
 		else {
 			retval = HTTP_UNAUTH;
@@ -332,6 +341,7 @@ static void freeNotifInfo(notifInfo *ni){
 	}
 	if (ni->conditions_text) free(ni->conditions_text);
 	if (ni->owner) free(ni->owner);
+	if (ni->jobid) free(ni->jobid);
 }
 
 static int getJobsRSS(edg_wll_Context ctx, char *feedType, edg_wll_JobStat **statesOut){
@@ -426,6 +436,33 @@ static int getJobsRSS(edg_wll_Context ctx, char *feedType, edg_wll_JobStat **sta
 	return 0;
 }
 
+static int getVMsOnHost(edg_wll_Context ctx, char *hostname, edg_wll_JobStat **statesOut){
+        edg_wlc_JobId *jobsOut;
+        edg_wll_QueryRec **conds;
+        int i;
+
+                conds = malloc(2*sizeof(*conds));
+                conds[0] = malloc(2*sizeof(**conds));
+                conds[0][0].attr = EDG_WLL_QUERY_ATTR_DESTINATION;
+                conds[0][0].op = EDG_WLL_QUERY_OP_EQUAL;
+                conds[0][0].value.c = hostname;
+                conds[0][1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
+                conds[1] = NULL;
+	
+
+        if (edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)conds, 0, &jobsOut, statesOut)){
+                *statesOut = NULL;
+        }
+
+        for (i = 0; conds[i]; i++)
+                free(conds[i]);
+        free(conds);
+
+        return 0;
+}
+
+
+
 static void hup_handler(int sig) {
 	purge_quit = 1;
 }
@@ -447,6 +484,172 @@ static char *glite_location() {
 	return location;
 }
 
+int edg_wll_ParseQueryConditions(edg_wll_Context ctx, const char *query, edg_wll_QueryRec ***conditions) {
+	edg_wll_QueryRec  **conds = NULL;
+	char *q = glite_lbu_UnescapeURL(query);
+	char *vartok, *vartok2, *cond, *attribute, *op, *operator, *value, *orvals, *errmsg = NULL;
+	int len, i=0, j, attr;
+	edg_wll_ErrorCode err = 0;
+
+	glite_common_log(LOG_CATEGORY_LB_SERVER_REQUEST, LOG_PRIORITY_DEBUG, "Query over HTML \"%s\"", q);
+
+	for( cond = strtok_r(q, "&", &vartok); cond ; cond = strtok_r(NULL, "&", &vartok) ) {
+		conds=(edg_wll_QueryRec**)realloc(conds, sizeof(edg_wll_QueryRec*) * (i+2));
+		conds[i] = NULL;
+		conds[i+1] = NULL;
+
+
+		len = strcspn(cond, "=<>");
+		if (len == strlen(cond)) {
+			asprintf(&errmsg, "Missing operator in condition \"%s\"", cond);
+			err = edg_wll_SetError(ctx, EINVAL, errmsg);
+			goto err;
+		}
+		attribute=(char*)calloc((len+1),sizeof(char));
+		strncpy(attribute, cond, len);
+		orvals=cond+len;
+		
+		for (attr=1; attr<EDG_WLL_QUERY_ATTR__LAST && strcasecmp(attribute,edg_wll_QueryAttrNames[attr]); attr++);
+		if (attr == EDG_WLL_QUERY_ATTR__LAST) {
+			asprintf(&errmsg, "Unknown argument \"%s\" in query", attribute);
+			err = edg_wll_SetError(ctx, EINVAL, errmsg);
+			goto err;
+		} // No else here. Don't worry, attribute will be assigned to the query structure later
+
+		j=0;
+		for( op = strtok_r(orvals, "|", &vartok2); op ; op = strtok_r(NULL, "|", &vartok2) ) {
+			if (strlen(op) == 0) continue;
+			conds[i]=(edg_wll_QueryRec*)realloc(conds[i], sizeof(edg_wll_QueryRec) * (j+2));
+			conds[i][j].value.c = NULL;
+			conds[i][j+1].attr = EDG_WLL_QUERY_ATTR_UNDEF;
+
+			conds[i][j].attr = (edg_wll_QueryAttr)attr;
+
+			len = strspn(op, "=<>");
+			if (len == 0) {
+				asprintf(&errmsg, "Missing operator before \"%s\"", op);
+				err = edg_wll_SetError(ctx, EINVAL, errmsg);
+				goto err;
+			}
+			operator = (char*)calloc((len+1),sizeof(char));
+			strncpy(operator, op, len);
+			value=op+len;
+			if (strlen(value) == 0) {
+				asprintf(&errmsg, "No value given for attribute \"%s\"", attribute);
+				err = edg_wll_SetError(ctx, EINVAL, errmsg);
+				goto err;
+			}
+
+			if (!strcmp(operator, "<")) conds[i][j].op = EDG_WLL_QUERY_OP_LESS;
+			else if (!strcmp(operator, ">")) conds[i][j].op = EDG_WLL_QUERY_OP_GREATER;
+			else if (!strcmp(operator, "=")) conds[i][j].op = EDG_WLL_QUERY_OP_EQUAL;
+			else if ((!strcmp(operator, "<>"))||(!strcmp(operator, "><"))) conds[i][j].op = EDG_WLL_QUERY_OP_UNEQUAL;
+			else {
+				asprintf(&errmsg, "Unknown operator \"%s\" in query", operator);
+				err = edg_wll_SetError(ctx, EINVAL, errmsg);
+				goto err;
+			}
+
+			switch(attr) {
+//				case EDG_WLL_QUERY_ATTR_LEVEL:
+//				case EDG_WLL_QUERY_ATTR_EVENT_TYPE:
+//				case EDG_WLL_QUERY_ATTR_CHKPT_TAG:
+				case EDG_WLL_QUERY_ATTR_JOBID:
+				case EDG_WLL_QUERY_ATTR_PARENT:
+					if ( glite_jobid_parse(value, (glite_jobid_t *)&conds[i][j].value.j) ) {
+						asprintf(&errmsg, "Cannot parse jobid \"%s\" in query", value);
+						err = edg_wll_SetError(ctx, EINVAL, errmsg);
+						goto err;
+					}
+					break;
+				case EDG_WLL_QUERY_ATTR_OWNER:
+				case EDG_WLL_QUERY_ATTR_LOCATION:
+				case EDG_WLL_QUERY_ATTR_DESTINATION:
+				case EDG_WLL_QUERY_ATTR_HOST:
+				case EDG_WLL_QUERY_ATTR_INSTANCE:
+				case EDG_WLL_QUERY_ATTR_USERTAG:
+				case EDG_WLL_QUERY_ATTR_JDL_ATTR:
+				case EDG_WLL_QUERY_ATTR_NETWORK_SERVER:
+				case EDG_WLL_QUERY_ATTR_SOURCE:
+					conds[i][j].value.c = strdup(value);
+					break;
+				case EDG_WLL_QUERY_ATTR_STATUS:
+					if ( 0 > (conds[i][j].value.i = edg_wll_StringToStat(value))) {
+						asprintf(&errmsg, "Unknown job state \"%s\" in query", value);
+						err = edg_wll_SetError(ctx, EINVAL, errmsg);
+						goto err;
+					}
+					break;
+				case EDG_WLL_QUERY_ATTR_DONECODE:
+				case EDG_WLL_QUERY_ATTR_EXITCODE:
+				case EDG_WLL_QUERY_ATTR_RESUBMITTED:
+					conds[i][j].value.i = strtol(value, NULL, 10);
+					switch (errno) {
+						case EINVAL: 
+							asprintf(&errmsg, "Cannot parse numeric value \"%s\" for attribute %s in query", value, attribute);
+							err = edg_wll_SetError(ctx, EINVAL, errmsg);
+	                                                goto err;
+						case ERANGE: 
+							asprintf(&errmsg, "Numeric value \"%s\" for attribute %s in query exceeds range", value, attribute);
+							err = edg_wll_SetError(ctx, EINVAL, errmsg);
+	                                                goto err;
+					}
+					break;
+				case EDG_WLL_QUERY_ATTR_TIME:
+				case EDG_WLL_QUERY_ATTR_STATEENTERTIME:
+				case EDG_WLL_QUERY_ATTR_LASTUPDATETIME:
+					conds[i][j].value.t.tv_sec = strtol(value, NULL, 10);
+					switch (errno) {
+						case EINVAL: 
+							asprintf(&errmsg, "Cannot parse numeric value \"%s\" for attribute %s in query", value, attribute);
+							err = edg_wll_SetError(ctx, EINVAL, errmsg);
+	                                                goto err;
+						case ERANGE: 
+							asprintf(&errmsg, "Numeric value \"%s\" for attribute %s in query exceeds range", value, attribute);
+							err = edg_wll_SetError(ctx, EINVAL, errmsg);
+	                                                goto err;
+					}
+					conds[i][j].value.t.tv_usec = 0;
+					break;
+				case EDG_WLL_QUERY_ATTR_JOB_TYPE:
+					if ( 0 > (conds[i][j].value.i = edg_wll_JobtypeStrToCode(value))) {
+						asprintf(&errmsg, "Unknown job type \"%s\" in query", value);
+						err = edg_wll_SetError(ctx, EINVAL, errmsg);
+						goto err;
+					}
+					break;
+				default:
+					asprintf(&errmsg, "Value conversion for attribute \"%s\" not supported in current implementation", attribute);
+					err = edg_wll_SetError(ctx, ENOSYS, errmsg);
+					goto err;
+					break;
+			}
+			free(operator);
+			j++;
+		}
+
+		free(attribute);	
+		i++;
+	}
+
+err:
+	if (err) {
+		if (conds) {
+			for (j = 0; conds[j]; j++) {
+				for (i = 0 ; (conds[j][i].attr != EDG_WLL_QUERY_ATTR_UNDEF); i++ )
+					edg_wll_QueryRecFree(&conds[j][i]);
+				free(conds[j]);
+			}
+			free(conds);
+		}
+	}
+	else *conditions=conds;
+
+	free(q);
+	free(errmsg);
+	return err;
+}
+
 edg_wll_ErrorCode edg_wll_ProtoV21(edg_wll_Context ctx,
 	char *request,char **headers,char *messageBody,
 	char **response,char ***headersOut,char **bodyOut)
@@ -641,14 +844,15 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 	char **response,char ***headersOut,char **bodyOut,
 	int *httpErr)
 {
-	char *requestPTR = NULL, *message = NULL, *requestMeat = NULL;
+	char *requestPTR = NULL, *message = NULL, *requestMeat = NULL, *querystr, *queryconds = NULL;
 	int	ret = HTTP_OK;
 	int 	html = outputHTML(headers);
 	int 	text = 0; //XXX: Plain text communication is special case of html here, hence when text=1, html=1 too
-	int	i;
+	int	i, j;
 	int	isadm;
 	http_admin_option adm_opt = HTTP_ADMIN_OPTION_MY;
 	http_extra_option extra_opt = HTTP_EXTRA_OPTION_NONE;
+	edg_wll_QueryRec **job_conditions = NULL;
 
 	edg_wll_ResetError(ctx);
 
@@ -701,6 +905,25 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 			if (check_request_for_query(requestPTR, "?all")) adm_opt = HTTP_ADMIN_OPTION_ALL;
 			else if (check_request_for_query(requestPTR, "?foreign")) adm_opt = HTTP_ADMIN_OPTION_FOREIGN;
 
+			if ((querystr = strstr(requestPTR, "?query="))) {
+				int len = strcspn(querystr+strlen("?query="),"? \f\n\r\t\v");
+				if (len) {
+					extra_opt = HTTP_EXTRA_OPTION_QUERY;
+					queryconds = (char*)calloc((len+1),sizeof(char));
+					queryconds = strncpy(queryconds, querystr+strlen("?query="), len);
+
+					switch(edg_wll_ParseQueryConditions(ctx, queryconds, &job_conditions)) {
+						case 0:
+							break;
+						case EINVAL:
+							ret = HTTP_INVALID; goto err;
+						case ENOSYS:
+							ret = HTTP_NOTIMPL; goto err;
+						default:
+							ret = HTTP_BADREQ; goto err;
+					}
+				}
+			}
 			strip_request_of_queries(requestPTR);
 		}
 
@@ -714,14 +937,18 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 		}
 
 	/* GET /: Current User Jobs */
-		else if ((!strcmp(requestMeat, "/")) && (extra_opt == HTTP_EXTRA_OPTION_NONE)) {
+		else if ((!strcmp(requestMeat, "/")) && ((extra_opt == HTTP_EXTRA_OPTION_NONE) || ( extra_opt == HTTP_EXTRA_OPTION_QUERY))) {
                 	edg_wlc_JobId *jobsOut = NULL;
 			edg_wll_JobStat *statesOut = NULL;
 			int	i, flags;
-			
+
+			// XXX: query strings are now not recognized as flags. Needs modifying.
 			flags = (requestPTR[1]=='?') ? edg_wll_string_to_stat_flags(requestPTR + 2) : 0;
 
-			switch (edg_wll_UserJobsServer(ctx, EDG_WLL_STAT_CHILDREN, &jobsOut, &statesOut)) {
+			switch ( extra_opt == HTTP_EXTRA_OPTION_QUERY ?
+                                edg_wll_QueryJobsServer(ctx, (const edg_wll_QueryRec **)job_conditions, flags, &jobsOut, &statesOut) :
+                                edg_wll_UserJobsServer(ctx, EDG_WLL_STAT_CHILDREN, &jobsOut, &statesOut)) {
+
 				case 0: edg_wll_UserInfoToHTML(ctx, jobsOut, statesOut, &message, text);
 					edg_wll_ServerStatisticsIncrement(ctx, text ? SERVER_STATS_TEXT_VIEWS : SERVER_STATS_HTML_VIEWS);
 					break;
@@ -745,6 +972,7 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 		else if (strcmp(requestMeat, "/RSS:")
 			&& strcmp(requestMeat, "/NOTIF")
 			&& strncmp(requestMeat, "/NOTIF:", 7)
+			&& strncmp(requestMeat, "/VMHOST:", 8)
 			&& strcmp(requestMeat, "/favicon.ico")
 			&& extra_opt == HTTP_EXTRA_OPTION_NONE
 			) {
@@ -880,6 +1108,22 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 				edg_wll_RSSFeed(ctx, states, requestPTR, &message);
 				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_RSS_VIEWS);
 			}
+	/*GET /VMHOST:[hostname] all VMs associated with hostname */
+		} else if (strncmp(requestMeat, "/VMHOST:", strlen("/VMHOST:")) == 0) {
+			char *host = requestMeat + strlen("/VMHOST:");
+			edg_wll_JobStat *states;
+			if (getVMsOnHost(ctx, host, &states)){
+				ret = HTTP_INTERNAL;
+                                goto err;
+			}
+			if (text){
+//				edg_wll_VMHostToText(ctx, requestPTR + strlen("/VMHOST:"), states, &message);
+//				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_TEXT_VIEWS);
+			}
+			else{
+				edg_wll_VMHostToHTML(ctx, host, states, &message);
+				edg_wll_ServerStatisticsIncrement(ctx, SERVER_STATS_HTML_VIEWS);
+			}
 		} else if (!strcmp(requestMeat, "/favicon.ico")) {
 			message=NULL;
 			ret = HTTP_NOTFOUND;
@@ -944,7 +1188,7 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 	
 		if (!strncmp(requestPTR,KEY_QUERY_EVENTS,sizeof(KEY_QUERY_EVENTS)-1)) { 
         	        edg_wll_Event *eventsOut = NULL;
-			edg_wll_QueryRec **job_conditions = NULL, **event_conditions = NULL;
+			edg_wll_QueryRec **event_conditions = NULL;
 			int i,j;
 
         	        if (parseQueryEventsRequest(ctx, messageBody, &job_conditions, &event_conditions)) 
@@ -974,15 +1218,6 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 						ret = HTTP_INTERNAL;
 			}
 
-			if (job_conditions) {
-	                        for (j = 0; job_conditions[j]; j++) {
-	                                for (i = 0 ; (job_conditions[j][i].attr != EDG_WLL_QUERY_ATTR_UNDEF); i++ )
-        	                                edg_wll_QueryRecFree(&job_conditions[j][i]);
-                	                free(job_conditions[j]);
-                        	}
-	                        free(job_conditions);
-        	        }
-
 	                if (event_conditions) {
         	                for (j = 0; event_conditions[j]; j++) {
                 	                for (i = 0 ; (event_conditions[j][i].attr != EDG_WLL_QUERY_ATTR_UNDEF); i++ )
@@ -1431,7 +1666,7 @@ edg_wll_ErrorCode edg_wll_Proto(edg_wll_Context ctx,
 	} else ret = HTTP_NOTALLOWED;
 
 err:	asprintf(response,"HTTP/1.1 %d %s",ret,edg_wll_HTTPErrorMessage(ret));
-	*headersOut = (char **) (html ? response_headers_html : response_headers_dglb);
+	*headersOut = (char **) (html ? (text ? response_headers_plain : response_headers_html) : response_headers_dglb);
 	if ((ret != HTTP_OK && ret != HTTP_ACCEPTED) && text)
                 *bodyOut = edg_wll_ErrorToText(ctx,ret);
 	else if ((ret != HTTP_OK && ret != HTTP_ACCEPTED) && html)
@@ -1439,6 +1674,17 @@ err:	asprintf(response,"HTTP/1.1 %d %s",ret,edg_wll_HTTPErrorMessage(ret));
 	else
 		*bodyOut = message;
 
+	if (job_conditions) {
+		for (j = 0; job_conditions[j]; j++) {
+			for (i = 0 ; (job_conditions[j][i].attr != EDG_WLL_QUERY_ATTR_UNDEF); i++ )
+				edg_wll_QueryRecFree(&job_conditions[j][i]);
+			free(job_conditions[j]);
+		}
+		free(job_conditions);
+	}
+
+	free(queryconds);
+
 	if (requestPTR) free(requestPTR);
 	if (requestMeat) free(requestMeat);
 
