@@ -13,8 +13,6 @@ typedef struct _edg_wll_server_statistics{
 	time_t start;
 } edg_wll_server_statistics;
 
-#define SERVER_STATISTICS_COUNT 15
-
 enum edg_wll_server_statistics_type{
 	SERVER_STATS_GLITEJOB_REGS = 0,
 	SERVER_STATS_PBSJOB_REGS,
@@ -30,8 +28,44 @@ enum edg_wll_server_statistics_type{
 	SERVER_STATS_NOTIF_LEGACY_SENT,
 	SERVER_STATS_NOTIF_MSG_SENT,
 	SERVER_STATS_WS_QUERIES,
-	SERVER_STATS_LBPROTO
+	SERVER_STATS_LBPROTO,
+	SERVER_STATISTICS_COUNT
 };
+// Add new values in front of LAST and do not change order
+
+static const char *edg_wll_server_statistics_type_title[] = {
+	"gLite job regs",
+	"PBS job regs",
+	"Condor job regs",
+	"CREAM job regs",
+	"Sandbox regs",
+	"Job events",
+	"HTML accesses",
+	"Plain text accesses",
+	"RSS accesses",
+	"Notification regs (legacy interface)",
+	"Notification regs (msg interface)",
+	"Notifications sent (legacy)",
+	"Notifications sent (msg)",
+	"WS queries",
+	"L&B protocol accesses" };
+
+static const char *edg_wll_server_statistics_type_key[] = {
+	"glite_jobs",
+	"pbs_jobs",
+	"condor_jobs",
+	"cream_jobs",
+	"sb_ft_jobs",
+	"events",
+	"queries_html",
+	"queries_text",
+	"queries_rss",
+	"notif_regs_legacy",
+	"notif_regs_msg",
+	"notifs_legacy",
+	"notis_msg",
+	"queries_ws",
+	"queries_api" };
 
 int edg_wll_InitServerStatistics(edg_wll_Context ctx, char *prefix);
 
